(*------------------------------------------------------------*)
(*                        BBACC.INC                           *)
(*         Modul zur Manipulation von Bayer-Baeumen           *)
(*------------------------------------------------------------*)
(*      Eine Bayer-Baum-Seite auf dem Bildschirm ausgeben     *)
PROCEDURE write_page(seiten_nr: fileindex; seite: Page);

VAR nr: INTEGER;

BEGIN
  WriteLn;  Write('SEITE ', seiten_nr, ':');
  IF seiten_nr<>0 THEN BEGIN
    WriteLn(' (Vorgaenger=', seite.vg, ')');
    Write(line_dn, seite.nf[0], line_up);
    FOR nr := 1 TO seite.anz DO
      Write(seite.info[nr].stw, line_dn, seite.nf[nr], line_up);
      WriteLn
  END
  ELSE WriteLn('Wurzelseite: ', seite.vg);
  WriteLn;
END;
(*------------------------------------------------------------*)
(* Uebergebene Seite zusammenschieben, Eintrag "nr" loeschen. *)
PROCEDURE compress_page(VAR seite: Page; nr: INTEGER);

BEGIN
  WHILE nr<seite.anz DO BEGIN
    seite.info[nr] := seite.info[Succ(nr)];
    seite.nf[nr] := seite.nf[Succ(nr)];
    nr := Succ(nr)
  END;
  seite.anz := Pred(seite.anz)
END;
(*------------------------------------------------------------*)
(* Zwei Seiten zusammenfassen, Eintrag "info" mittig anordnen *)
PROCEDURE merge_pages(VAR seite_1: Page;
                      info: ref; seite_2: Page);
VAR nr: INTEGER;

BEGIN
  seite_1.nf[Succ(seite_1.anz)] := 0;
  seite_1.info[Succ(seite_1.anz)] := info;
  FOR nr := 1 TO seite_2.anz DO BEGIN
    seite_1.info[Succ(nr+seite_1.anz)] := seite_2.info[nr];
    seite_1.nf[Succ(nr+seite_1.anz)] := 0
  END;
  seite_1.anz := maxanz
END;
(*-------------------------------------------------------------*)
(* Angegebene Seite zusammen mit neuen Eintrag (info) und den  *)
(*    beiden Nachfolgern in eine Grossseite eintragen.         *)
PROCEDURE merge_to_bigpage(VAR gr_seite: bigpage; seite: Page;
                           nf_l: fileindex; info: ref;
                           nf_r: fileindex                    );
VAR nr: INTEGER;

BEGIN
  nr := 1;
  gr_seite.nf[0] := seite.nf[0];
  WHILE (info.stw>=seite.info[nr].stw) AND (nr<=maxanz) DO BEGIN
    gr_seite.info[nr] := seite.info[nr];
    gr_seite.nf[nr] := seite.nf[nr];
    nr := Succ(nr)
  END;
  gr_seite.nf[Pred(nr)] := nf_l;
  gr_seite.info[nr] := info;
  gr_seite.nf[nr] := nf_r;
  WHILE nr<=maxanz DO BEGIN
    gr_seite.info[Succ(nr)] := seite.info[nr];
    gr_seite.nf[Succ(nr)] := seite.nf[nr];
    nr := Succ(nr)
  END;
  gr_seite.vg := seite.vg
END;
(*-------------------------------------------------------------*)
(*            Grosseite in zwei normale Seiten u.              *)
(*             einen einzelnen Eintrag gespalten.              *)
PROCEDURE split_bigpage(gr_seite: bigpage;
                        VAR seite_l: Page;
                        VAR info: ref; VAR seite_r: Page);
VAR nr: INTEGER;

BEGIN
  seite_l.nf[0] := gr_seite.nf[0];
  FOR nr := 1 TO maxanz SHR 1 DO BEGIN
    seite_l.info[nr] := gr_seite.info[nr];
    seite_l.nf[nr] := gr_seite.nf[nr]
  END;
  seite_l.anz := maxanz SHR 1;  seite_l.vg := gr_seite.vg;
  nr := Succ(bt_order);  info := gr_seite.info[nr];
  seite_r.nf[0] := gr_seite.nf[nr];
  WHILE nr<=maxanz DO BEGIN
    nr := Succ(nr);
    seite_r.info[Pred(nr-bt_order)] := gr_seite.info[nr];
    seite_r.nf[Pred(nr-bt_order)] := gr_seite.nf[nr]
  END;
  seite_r.anz := maxanz SHR 1;  seite_r.vg := gr_seite.vg
END;
(*-------------------------------------------------------------*)
(* Eintrag mit den beiden Nachfolgern in Seite einsortieren.   *)
PROCEDURE bubble(VAR seite: Page; nf_l: fileindex;
                 info: ref; nf_r: fileindex);
VAR nr: INTEGER;

BEGIN
  seite.anz := Succ(seite.anz);  seite.info[seite.anz] := info;
  seite.nf[seite.anz] := nf_r;   nr := seite.anz;
  WHILE (nr>1) AND
        (seite.info[Pred(nr)].stw>seite.info[nr].stw) DO BEGIN
    info := seite.info[Pred(nr)];  nf_r := seite.nf[Pred(nr)];
    seite.info[Pred(nr)] := seite.info[nr];
    seite.nf[Pred(nr)] := seite.nf[nr];
    seite.info[nr] := info;  seite.nf[nr] := nf_r;
    nr := Pred(nr)
  END;
  seite.nf[Pred(nr)] := nf_l
END;
(*-------------------------------------------------------------*)
(* D. zur "seite" passende Nachbarseite und d. Position des    *)
(* direkten Nachbarschluessels (trenn_nr, Schluessel selbst in *)
(* trenn_info), die Richtung (-1 = vorher, +1 = nachher) der   *)
(* Nachbarseite liegt und die dazwischen liegende "vorseite"   *)
(* zurueckgeben. Falls kein Nachbar frei, wird dies in         *)
(* "vorhanden" vermerkt.                                       *)
PROCEDURE neighbour(VAR seite: Page; VAR seiten_nr: fileindex;
                    VAR vorhanden: BOOLEAN;
                    VAR delta_nf, delta_info: delta;
                    VAR vorseite: Page;
                    VAR vorseiten_nr: fileindex;
                    VAR trenn_info: ref; VAR trenn_nr: INTEGER;
                    VAR nachbarseite: Page;
                    VAR nachbarseiten_nr: fileindex;
                    VAR nachbar_nr: INTEGER);

BEGIN
  vorseiten_nr := seite.vg;
  get_page(vorseiten_nr, vorseite);
  trenn_nr := 0;
  WHILE vorseite.nf[trenn_nr]<>seiten_nr DO
    trenn_nr := Succ(trenn_nr);
  IF trenn_nr=0 THEN BEGIN
    delta_nf := +1; delta_info := 1
  END
  ELSE BEGIN
    delta_nf := -1; delta_info := 0
  END;
  nachbarseiten_nr := vorseite.nf[trenn_nr+delta_nf];
  get_page(nachbarseiten_nr, nachbarseite);
  vorhanden := nachbarseite.anz>bt_order
END;
(*-------------------------------------------------------------*)
(*               Zwei Seiten austauschen.                      *)
PROCEDURE swap_pages(VAR seite1: Page; VAR s_nr1: fileindex;
                     VAR seite2: Page; VAR s_nr2: fileindex);
VAR seite0: Page;
    s_nr0: fileindex;

BEGIN
  seite0 := seite1; seite1 := seite2; seite2 := seite0;
  s_nr0 := s_nr1; s_nr1 := s_nr2; s_nr2 := s_nr0
END;
(*-------------------------------------------------------------*)
(*          Eintrag "nr" in der Blattseite loeschen.           *)
PROCEDURE delete_key_in_leaf(VAR seite: Page;
                             VAR seiten_nr: fileindex;
                             VAR nr: INTEGER);
VAR vorhanden: BOOLEAN;
    vorseite, nachbarseite: Page;
    vorseiten_nr, nachbarseiten_nr: fileindex;
    trenn_nr, nachbar_nr: INTEGER;
    trenn_info: ref;
    delta_nf, delta_info: delta;

(* Aus einer Nachbarseite einen Eintrag leihen, er kommt selbst *)
(* in die Vorseite, der dort stehende Eintrag in die zu klein   *)
(* gewordene Seite.*)
PROCEDURE Lend_key;
BEGIN
  IF delta_nf<0 THEN nachbar_nr := nachbarseite.anz
  ELSE nachbar_nr := 1;
  bubble(seite, 0, vorseite.info[trenn_nr+delta_info], 0);
  vorseite.info[trenn_nr+delta_info] :=
    nachbarseite.info[nachbar_nr];
  put_page(seiten_nr, seite);
  trenn_nr := trenn_nr+delta_info;
  trenn_info := vorseite.info[trenn_nr];
  put_page(vorseiten_nr, vorseite);
  seite := nachbarseite; seiten_nr := nachbarseiten_nr;
  nr := nachbar_nr;
END (*OF PROCEDURE lend_key*);

(* Bei Seitenzusammenfassung redundant verbleibende Eintrag    *)
(* in der Vorseite loeschen, Sonderfaelle beruecksichtigen.    *)
PROCEDURE delete_separator;
BEGIN
  IF vorseite.vg=0 THEN BEGIN (*Vorseite ist Wurzelseite*)
    IF vorseite.anz=1 THEN BEGIN (* Letzte Eintrag loeschen *)
      remove_page(vorseite, vorseiten_nr);
      vorseite.vg := vorseite.nf[0];
      put_page(0, vorseite); seite.vg := 0;
      put_page(seiten_nr, seite)
    END
    ELSE compress_page(vorseite, trenn_nr);
    seiten_nr := -1
  END
  ELSE BEGIN (* Die Vorseite ist eine innere Seite *)
    IF vorseite.anz>bt_order  THEN BEGIN (* wird nicht zu klein *)
       compress_page(vorseite, trenn_nr); seiten_nr := -1
    END
    ELSE BEGIN (* sie wird zu klein *)
      seite := vorseite;  seiten_nr := vorseiten_nr;
      nr := trenn_nr
    END
  END
END; (*OF PROCEDURE delete_separator*)

(* Zwei Seiten zusammenfassen, ggf. Reihenfolge korrigieren.   *)
PROCEDURE merge;
BEGIN
  IF delta_nf<0  THEN BEGIN
    swap_pages(seite, seiten_nr, nachbarseite, nachbarseiten_nr);
    vorseite.nf[trenn_nr+delta_nf] := seiten_nr;
    vorseite.nf[trenn_nr] := 0 (*In diesem Zweig nicht mehr...*)
  END                          (*...suchen.*)
  ELSE vorseite.nf[trenn_nr+delta_nf] := 0;  (*hier auch nicht*)
  trenn_nr := trenn_nr+delta_info;
  trenn_info := vorseite.info[trenn_nr];
  put_page(vorseiten_nr, vorseite);
  merge_pages(seite, trenn_info, nachbarseite);
  put_page(seiten_nr, seite);
  remove_page(nachbarseite, nachbarseiten_nr);
  delete_separator; put_page(vorseiten_nr, vorseite);
END (*OF PROCEDURE merge*);

(* PROCEDURE delete_in_leaf *)
BEGIN
  compress_page(seite, nr);
  IF seite.anz<bt_order THEN BEGIN (*Seite wurde zu klein*)
    IF seite.vg=0 THEN BEGIN (*Seite ist Wurzelseite*)
      IF seite.anz=0 THEN (* Baum leer? *)
        remove_page(seite, seiten_nr)
      ELSE (*Wurzelseite darf kleiner werden*)
        put_page(seiten_nr, seite);
      seiten_nr := -1 (*keine weiteren Loeschungen*)
    END
    ELSE BEGIN (*Seite ist nicht Wurzelseite*)
      neighbour(seite, seiten_nr, vorhanden,
                delta_nf, delta_info,
                vorseite, vorseiten_nr, trenn_info, trenn_nr,
                nachbarseite, nachbarseiten_nr, nachbar_nr);
      IF vorhanden THEN
        Lend_key (* Eintrag ausleihen (Rotation) *)
      ELSE merge     (* Seiten zusammenfassen *)
    END
  END
  ELSE BEGIN (* Seite hat ausreichende Groesse *)
    put_page(seiten_nr, seite);
    seiten_nr := -1 (*keine weiteren Loeschungen*)
  END
END;
(*-------------------------------------------------------------*)
(* Aus innerer Seite kann nicht geloescht werden. Solcher      *)
(* Schluessel muss durch Schluessel aus einem Blatt ersetzt    *)
(* werden, dessen alte Position ist zu loeschen.               *)
PROCEDURE replace_with_leafkey(VAR seite: Page; nr: INTEGER;
                               VAR blatt_seite: Page;
                               VAR blatt_snr: fileindex;
                               VAR nachbar_nr: INTEGER);
VAR alt_snr: fileindex;

BEGIN
  IF seite.nf[nr]>0 THEN BEGIN (*es existiert ein rechter Nachfolger*)
    blatt_snr := seite.nf[nr];
    REPEAT
      get_page(blatt_snr, blatt_seite);
      alt_snr := blatt_snr;
      blatt_snr := blatt_seite.nf[0]
    UNTIL blatt_snr=0;
    nachbar_nr := 1
  END
  ELSE BEGIN (*suche im linken Zweig*)
    blatt_snr := seite.nf[Pred(nr)];
    REPEAT
      get_page(blatt_snr, blatt_seite);
      alt_snr := blatt_snr;
      blatt_snr := blatt_seite.nf[blatt_seite.anz]
    UNTIL blatt_snr=0;
    nachbar_nr := blatt_seite.anz
  END;
  blatt_snr := alt_snr;
  seite.info[nr] := blatt_seite.info[nachbar_nr]
END;
(*-------------------------------------------------------------*)
(*            In Indexdatei nach Stichwort suchen.             *)
PROCEDURE search_key(such_stw: key;
                     VAR seite: Page; VAR seiten_nr: fileindex;
                     VAR nr: INTEGER; VAR gef: BOOLEAN);
VAR abbruch: BOOLEAN;

BEGIN
  get_page(0, seite);  seiten_nr := seite.vg;  gef := FALSE;
  abbruch := seiten_nr=0;
  WHILE NOT(gef OR abbruch) DO BEGIN
    get_page(seiten_nr, seite); nr := 1;
    WHILE (such_stw>seite.info[nr].stw) AND (nr<=seite.anz) DO
      nr := Succ(nr);
    IF (nr<=seite.anz) AND (such_stw=seite.info[nr].stw) THEN
      gef := TRUE
    ELSE BEGIN
      nr := Pred(nr);
      IF seite.nf[nr]<>0 THEN
        seiten_nr := seite.nf[nr] (*naechste Seite*)
      ELSE abbruch := TRUE  (*Stichwort ist nicht vorhanden*)
    END
  END
END;
(*-------------------------------------------------------------*)
(* Neuen Schluessel in Seite (seiten_nr) eintragen, diese muss *)
(* zuvor mit (search_key) ermittelt worden sein.               *)
PROCEDURE insert_key(info: ref; seiten_nr: fileindex);

VAR seite, neuseite, nullseite: Page;
    neuseiten_nr: fileindex;
    gr_seite: bigpage;
    nf_l, nf_r: fileindex;

BEGIN
  nf_l := 0; nf_r := 0;
  REPEAT
    IF seiten_nr=0 THEN BEGIN (*Erste Seite des Baums*)
      seite.anz := 1;  seite.vg := 0;  seite.nf[0] := nf_l;
      seite.info[1] := info;  seite.nf[1] := nf_r;
      generate_page(seite, seiten_nr);
      get_page(0, nullseite);  nullseite.vg := seiten_nr;
      put_page(0, nullseite);  seiten_nr := -1
    END
    ELSE BEGIN (*existierende Seite des Baums*)
      get_page(seiten_nr, seite);
      IF seite.anz<maxanz THEN BEGIN (*es ist noch Platz*)
        bubble(seite, nf_l, info, nf_r);
        put_page(seiten_nr, seite);
        seiten_nr := -1 (*keine weiteren Einfuegungen*)
      END
      ELSE BEGIN (*es ist kein Platz mehr*)
        merge_to_bigpage(gr_seite, seite, nf_l, info, nf_r);
        split_bigpage(gr_seite, seite, info, neuseite);
        put_page(seiten_nr, seite);
        generate_page(neuseite, neuseiten_nr);
        nf_l := seiten_nr;  nf_r := neuseiten_nr;
        seiten_nr := seite.vg      (*weiter Eintragen*)
      END
    END
  UNTIL seiten_nr<0
END;
(*-------------------------------------------------------------*)
(*         Aus Seite (seite) den Eintrag (nr) loeschen         *)
PROCEDURE delete_key(seite: Page; seiten_nr: fileindex;
                     nr: INTEGER                       );

VAR trenn_nr, nachbar_nr: INTEGER;
    nachbarseite, hilfsseite: Page;
    nachbarseiten_nr, hilfsseiten_nr: fileindex;
    vorhanden: BOOLEAN;

BEGIN
  REPEAT
    IF seite.nf[0]=0 THEN (*ist Seite eine Blattseite?*)
      delete_key_in_leaf(seite, seiten_nr, nr)
    ELSE BEGIN (*Loeschung in innerere Seite nicht direkt moeglich*)
      replace_with_leafkey(seite, nr, nachbarseite,
                           nachbarseiten_nr, nachbar_nr);
      put_page(seiten_nr, seite);
      delete_key_in_leaf(nachbarseite,nachbarseiten_nr,nachbar_nr);
      seite := nachbarseite;
      seiten_nr := nachbarseiten_nr;
      nr := nachbar_nr                    (*weiter loeschen*)
    END
  UNTIL seiten_nr<0
END;
(*-------------------------------------------------------------*)
(*   Ein Hauptdatensatz an der angegebenen Stelle speichern.   *)
PROCEDURE put_rec(satz_nr: fileindex; VAR datensatz: rec);
BEGIN
  Seek(datei, satz_nr);  Write(datei, datensatz)
END;
(*-------------------------------------------------------------*)
(*  Einen Hauptdatensatz von der angegebenen Position lesen.   *)
PROCEDURE get_rec(satz_nr: fileindex; VAR datensatz: rec);
BEGIN
  Seek(datei, satz_nr);  Read(datei, datensatz)
END;
(*-------------------------------------------------------------*)
(*  Neuen Hauptspeicherplatz anlegen und Datensatz speichern.  *)
PROCEDURE generate_rec(VAR datensatz: rec; VAR satz_nr: fileindex);

VAR fs_nr: fileindex;

BEGIN
  Seek(freisaetze, 0); satz_nr := -1;
  WHILE NOT(Eof(freisaetze)) AND (satz_nr<0) DO
    Read(freisaetze, satz_nr);
  IF satz_nr<0 THEN satz_nr := FileSize(datei)
  ELSE BEGIN
    fs_nr := -1;
    Seek(freisaetze, FilePos(freisaetze)-1);
    Write(freisaetze, fs_nr)
  END;
  put_rec(satz_nr, datensatz)
END;
(*-------------------------------------------------------------*)
(*       Angegebenen Speicherplatz wieder freigeben.           *)
PROCEDURE remove_rec(satz_nr: fileindex);

VAR datensatz: rec;
    fs_nr: fileindex;

BEGIN
  get_rec(satz_nr, datensatz);  datensatz.stw := '@';
  (*damit er bei seriellem Durchlauf erkannt wird*)
  put_rec(satz_nr, datensatz);  Seek(freisaetze, 0);
  fs_nr := -1;
  WHILE NOT(Eof(freisaetze)) AND (fs_nr<0) DO
     Read(freisaetze, fs_nr);
  IF fs_nr>=0 THEN Seek(freisaetze, FilePos(freisaetze)-1);
  Write(freisaetze, satz_nr)
END;
(*-------------------------------------------------------------*)
(*        Schluessel suchen und Datensatz ausgeben.            *)
PROCEDURE search_rec(schluessel: key; VAR vorhanden: BOOLEAN);

VAR seite: Page;
    seiten_nr: fileindex;
    nr: INTEGER;
    datensatz: rec;

BEGIN
  search_key(schluessel, seite, seiten_nr, nr, vorhanden);
  IF vorhanden THEN BEGIN
   get_rec(seite.info[nr].zeiger, datensatz);
   write_rec(datensatz)
  END
END;
(*-------------------------------------------------------------*)
(*              Schluessel des Datensatz suchen,               *)
(*        wenn nicht vorhanden, Datensatz abspeichern.         *)
PROCEDURE insert_rec(datensatz: rec; VAR vorhanden: BOOLEAN);

VAR seite: Page;
    seiten_nr, satz_nr: fileindex;
    nr: INTEGER;
    info: ref;

BEGIN
  search_key(datensatz.stw, seite, seiten_nr, nr, vorhanden);
  IF NOT(vorhanden) THEN BEGIN
    generate_rec(datensatz, satz_nr);
    info.stw := datensatz.stw; info.zeiger := satz_nr;
    insert_key(info, seiten_nr)
  END
END;
(*-------------------------------------------------------------*)
(*   Schluessel suchen, Datensatz wenn vorhanden abspeichern   *)
PROCEDURE delete_rec(schluessel: key; VAR vorhanden: BOOLEAN);

VAR seite: Page;
    seiten_nr: fileindex;
    nr: INTEGER;
    datensatz: rec;

BEGIN
  search_key(schluessel, seite, seiten_nr, nr, vorhanden);
  IF vorhanden THEN BEGIN
    remove_rec(seite.info[nr].zeiger);
    delete_key(seite, seiten_nr, nr)
  END
END;

(*-------------------------------------------------------------*)
(*                       Ende BBACC.INC                        *)